@@ -177,7 +177,7 @@ step3:
 
     match = 1;
     for (i = 0; match && i < strlen (IDNA_ACE_PREFIX); i++)
-      if (IDNA_ACE_PREFIX[i] != src[i])
+      if (((unsigned long)IDNA_ACE_PREFIX[i] & 0xFF) != src[i])
 	match = 0;
     if (match)
       return IDNA_CONTAINS_ACE_PREFIX;
@@ -342,7 +342,7 @@ idna_to_unicode (const unsigned long *in, size_t inlen,
 		 int allowunassigned, int usestd3asciirules)
 {
   int rc;
-  int outlensave = *outlen;
+  size_t outlensave = *outlen;
   char *p;
 
   p = stringprep_ucs4_to_utf8 (in, inlen, NULL, NULL);
@@ -358,8 +358,8 @@ idna_to_unicode (const unsigned long *in, size_t inlen,
 				 p, BUFSIZ);
   if (rc != IDNA_SUCCESS)
     {
-      memcpy (out, in,
-	      sizeof (in[0]) * (inlen < outlensave ? inlen : outlensave));
+      memcpy (out, in, sizeof (in[0]) * (inlen < outlensave ?
+					 inlen : outlensave));
       *outlen = inlen;
     }
 
@@ -448,7 +448,7 @@ int
 idna_utf8_to_ace (const char *input, char **output)
 {
   unsigned long *ucs4;
-  int ucs4len;
+  size_t ucs4len;
   int rc;
 
   ucs4 = stringprep_utf8_to_ucs4 (input, -1, &ucs4len);
@@ -586,7 +586,7 @@ int
 idna_utf8ace_to_ucs4 (const char *input, unsigned long **output)
 {
   unsigned long *ucs4;
-  int ucs4len;
+  size_t ucs4len;
   int rc;
 
   ucs4 = stringprep_utf8_to_ucs4 (input, -1, &ucs4len);
