@@ -118,10 +118,8 @@ Stringprep_profile stringprep_nameprep[] = {
   ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_7, "C.7"}
   ,
-  /* Already done below for bidi:
-     {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
-     ,
-   */
+  {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
+  ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_9, "C.9"}
   ,
   {STRINGPREP_BIDI, 0, 0, "BIDI"}
@@ -162,10 +160,8 @@ Stringprep_profile stringprep_kerberos5[] = {
   ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_7, "C.7"}
   ,
-  /* Already done below for bidi:
-     {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
-     ,
-   */
+  {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
+  ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_9, "C.9"}
   ,
   {STRINGPREP_BIDI, 0, 0, "BIDI"}
@@ -199,6 +195,8 @@ Stringprep_table_element stringprep_xmpp_nodeprep_prohibit[] = {
   {0x00003E}
   ,				/* #x3E (>) */
   {0x000040}			/* #x40 (@) */
+  ,
+  {0}
 };
 
 Stringprep_profile stringprep_xmpp_nodeprep[] = {
@@ -226,10 +224,8 @@ Stringprep_profile stringprep_xmpp_nodeprep[] = {
   ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_7, "C.7"}
   ,
-  /* Already done below for bidi:
-     {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
-     ,
-   */
+  {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
+  ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_9, "C.9"}
   ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_xmpp_nodeprep_prohibit,
@@ -274,10 +270,8 @@ Stringprep_profile stringprep_xmpp_resourceprep[] = {
   ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_7, "C.7"}
   ,
-  /* Already done below for bidi:
-     {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
-     ,
-   */
+  {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_8, "C.8"}
+  ,
   {STRINGPREP_PROHIBIT_TABLE, 0, stringprep_generic_C_9, "C.9"}
   ,
   {STRINGPREP_BIDI, 0, 0, "BIDI"}
