@@ -1,5 +1,5 @@
 /* stringprep.c		Core stringprep implementation.
- * Copyright (C) 2002  Simon Josefsson
+ * Copyright (C) 2002, 2003  Simon Josefsson
  *
  * This file is part of GNU Libidn.
  *
@@ -21,8 +21,6 @@
 
 #include "internal.h"
 
-#define DBG 0
-
 static int
 stringprep_find_character_in_table (unsigned long ucs4,
 				    Stringprep_table_element * table)
@@ -37,9 +35,9 @@ stringprep_find_character_in_table (unsigned long ucs4,
   return -1;
 }
 
-static int
+static size_t
 stringprep_find_string_in_table (unsigned long *ucs4,
-				 int ucs4len,
+				 size_t ucs4len,
 				 int *tablepos,
 				 Stringprep_table_element * table)
 {
@@ -58,33 +56,66 @@ stringprep_find_string_in_table (unsigned long *ucs4,
 
 static int
 stringprep_apply_table_to_string (unsigned long *ucs4,
-				  int *ucs4len,
-				  int maxucs4len,
-				  Stringprep_table_element * table)
+				  size_t *ucs4len,
+				  size_t maxucs4len,
+				  Stringprep_table_element * table,
+				  const char *tablename)
 {
   int i;
-  int pos, maplen;
+  size_t pos, maplen;
+#ifdef DRAFT
+  int modified = 0;
+#endif
 
   while ((pos = stringprep_find_string_in_table
 	  (ucs4, *ucs4len, &i, table)) != -1)
     {
-#if DBG
-      printf ("hit %06lx %06lx-%06lx\n", ucs4[pos], table[i].start,
-	      table[i].end);
-#endif
-
       for (maplen = 0; table[i].map[maplen]; maplen++)
 	;
 
       if (*ucs4len - 1 + maplen >= maxucs4len)
 	return STRINGPREP_TOO_SMALL_BUFFER;
 
+#ifdef DRAFT
+      modified = 1;
+
+      if (table[i].end)
+	printf ("Table %s maps U+%04lx (in range %04lx-%04lx) to",
+		tablename, ucs4[pos], table[i].start, table[i].end);
+      else
+	printf ("Table %s maps U+%04lx to", tablename, ucs4[pos]);
+      if (maplen == 0)
+	printf(" nothing");
+      else
+	{
+	  int n;
+
+	  for (n = 0; n < maplen; n++)
+	    printf(" U+%04lx", table[i].map[n]);
+	}
+      printf(".\n");
+#endif
+
       memmove (&ucs4[pos + maplen], &ucs4[pos + 1],
 	       *ucs4len * sizeof (unsigned long) - (&ucs4[pos + 1] - ucs4));
       memcpy (&ucs4[pos], table[i].map, sizeof (unsigned long) * maplen);
       *ucs4len = *ucs4len - 1 + maplen;
     }
 
+#ifdef DRAFT
+  if (modified)
+    {
+      int j;
+      for (j = 0; j < *ucs4len; j++)
+	{
+	  printf ("U+%04lx ", ucs4[j]);
+	  if ((j+1)%8 == 0)
+	    printf("\n\t");
+	}
+      printf ("\n");
+    }
+#endif
+
   return STRINGPREP_OK;
 }
 
@@ -118,15 +149,14 @@ stringprep_apply_table_to_string (unsigned long *ucs4,
  * Return value: Returns 0 iff successful, or an error code.
  **/
 int
-stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
+stringprep (char *in, size_t maxlen, int flags, Stringprep_profile * profile)
 {
-  int i;
+  int i, j;
   int rc;
   char *p = 0;
   unsigned long *q = 0;
   unsigned long *ucs4;
-  int ucs4len;
-  int maxucs4len;
+  size_t ucs4len, maxucs4len;
 
   ucs4 = stringprep_utf8_to_ucs4 (in, -1, &ucs4len);
   maxucs4len = 4 * ucs4len + 10;	/* XXX */
@@ -137,12 +167,17 @@ stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
       goto done;
     }
 
-#if DBG
+#ifdef DRAFT
   {
     int j;
-    printf ("\t;;    in: ");
+    printf ("input: ");
     for (j = 0; j < ucs4len; j++)
-      printf ("U+%06lx ", ucs4[j]);
+      {
+	printf ("U+%04lx ", ucs4[j]);
+	if ((j+1)%8 == 0)
+	  printf("\n\t");
+      }
+    printf ("\n");
     printf ("\n");
   }
 #endif
@@ -153,7 +188,13 @@ stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
 	{
 	case STRINGPREP_NFKC:
 	  if (UNAPPLICAPLEFLAGS (flags, profile[i].flags))
-	    break;
+	    {
+#ifdef DRAFT
+	      printf("Unicode normalization with form KC not used.\n");
+#endif
+	      break;
+	    }
+
 	  if (flags & STRINGPREP_NO_NFKC && !profile[i].flags)
 	    {
 	      /* Profile requires NFKC, but callee asked for no NFKC. */
@@ -169,18 +210,39 @@ stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
 	      goto done;
 	    }
 
-	  for (ucs4len = 0; ucs4[ucs4len]; ucs4len++)
+	  for (j = 0; q[j]; j++)
 	    ;
 
+#ifdef DRAFT
+	  if (ucs4len != j || memcmp(ucs4, q, sizeof(ucs4[0]) * ucs4len) != 0)
+	    {
+	      int n;
+	      printf("Unicode normalization with form KC maps string into:\n");
+	      for (n = 0; n < j; n++)
+		{
+		  printf ("U+%04lx ", q[n]);
+		  if ((n+1)%8 == 0)
+		    printf("\n\t");
+		}
+	      printf ("\n");
+	    }
+#endif
+
 	  free (ucs4);
 	  ucs4 = q;
+	  ucs4len = j;
 	  q = 0;
 	  break;
 
 	case STRINGPREP_PROHIBIT_TABLE:
-	  if (stringprep_find_string_in_table
-	      (ucs4, ucs4len, NULL, profile[i].table) != -1)
+	  j = stringprep_find_string_in_table (ucs4, ucs4len,
+					       NULL, profile[i].table);
+	  if (j != -1)
 	    {
+#ifdef DRAFT
+	      printf("Table %s prohibits string (character U+%04lx).\n",
+		     profile[i].name, ucs4[j]);
+#endif
 	      rc = STRINGPREP_CONTAINS_PROHIBITED;
 	      goto done;
 	    }
@@ -190,19 +252,26 @@ stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
 	  if (UNAPPLICAPLEFLAGS (flags, profile[i].flags))
 	    break;
 	  if (flags & STRINGPREP_NO_UNASSIGNED)
-	    if (stringprep_find_string_in_table
-		(ucs4, ucs4len, NULL, profile[i].table) != -1)
-	      {
-		rc = STRINGPREP_CONTAINS_UNASSIGNED;
-		goto done;
+	    {
+	      j = stringprep_find_string_in_table
+		(ucs4, ucs4len, NULL, profile[i].table);
+	      if (j != -1)
+		{
+#ifdef DRAFT
+		  printf("Table %s prohibits string (unassigned "
+			 "character U+%04lx).\n", profile[i].name, ucs4[j]);
+#endif
+		  rc = STRINGPREP_CONTAINS_UNASSIGNED;
+		  goto done;
 	      }
+	    }
 	  break;
 
 	case STRINGPREP_MAP_TABLE:
 	  if (UNAPPLICAPLEFLAGS (flags, profile[i].flags))
 	    break;
 	  rc = stringprep_apply_table_to_string
-	    (ucs4, &ucs4len, maxucs4len, profile[i].table);
+	    (ucs4, &ucs4len, maxucs4len, profile[i].table, profile[i].name);
 	  if (rc != STRINGPREP_OK)
 	    goto done;
 	  break;
@@ -219,15 +288,21 @@ stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
 	    int done_l = 0;
 	    int contains_ral = -1;
 	    int contains_l = -1;
-	    int j;
+	    int k;
 
 	    for (j = 0; profile[j].operation; j++)
 	      if (profile[j].operation == STRINGPREP_BIDI_PROHIBIT_TABLE)
 		{
 		  done_prohibited = 1;
-		  if (stringprep_find_string_in_table
-		      (ucs4, ucs4len, NULL, profile[j].table) != -1)
+		  k = stringprep_find_string_in_table (ucs4, ucs4len,
+						       NULL, profile[j].table);
+		  if (k != -1)
 		    {
+#ifdef DRAFT
+		      printf("Table %s prohibits string "
+			     "(bidi, character U+%04lx).\n",
+			     profile[i].name, ucs4[j]);
+#endif
 		      rc = STRINGPREP_BIDI_CONTAINS_PROHIBITED;
 		      goto done;
 		    }
@@ -255,19 +330,28 @@ stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
 
 	    if (contains_ral != -1 && contains_l != -1)
 	      {
+#ifdef DRAFT
+		printf("String contains both L and RAL characters.\n");
+#endif
 		rc = STRINGPREP_BIDI_BOTH_L_AND_RAL;
 		goto done;
 	      }
 
 	    if (contains_ral != -1)
-	      if (!(stringprep_find_character_in_table
-		    (ucs4[0], profile[contains_ral].table) != -1 &&
-		    stringprep_find_character_in_table
-		    (ucs4[ucs4len - 1], profile[contains_ral].table) != -1))
-		{
-		  rc = STRINGPREP_BIDI_LEADTRAIL_NOT_RAL;
-		  goto done;
-		}
+	      {
+		if (!(stringprep_find_character_in_table
+		      (ucs4[0], profile[contains_ral].table) != -1 &&
+		      stringprep_find_character_in_table
+		      (ucs4[ucs4len - 1], profile[contains_ral].table) != -1))
+		  {
+#ifdef DRAFT
+		    printf("Bidi string does not start/end with "
+			   "RAL characters.\n");
+#endif
+		    rc = STRINGPREP_BIDI_LEADTRAIL_NOT_RAL;
+		    goto done;
+		  }
+	      }
 	  }
 	  break;
 
@@ -276,17 +360,23 @@ stringprep (char *in, int maxlen, int flags, Stringprep_profile * profile)
 	  goto done;
 	  break;
 	}
-#if DBG
-      {
-	int j;
-	printf ("\t;; %5s: ", profile[i].name ? profile[i].name : "<NONAME>");
-	for (j = 0; j < ucs4len; j++)
-	  printf ("U+%06lx ", ucs4[j]);
-	printf ("\n");
-      }
-#endif
     }
 
+#ifdef DRAFT
+  {
+    int j;
+    printf ("\n");
+    printf ("output: ");
+    for (j = 0; j < ucs4len; j++)
+      {
+	printf ("U+%04lx ", ucs4[j]);
+	if ((j+1)%8 == 0)
+	  printf("\n\t");
+      }
+    printf ("\n");
+  }
+#endif
+
   p = stringprep_ucs4_to_utf8 (ucs4, ucs4len, 0, 0);
 
   if (strlen (p) >= maxlen)
